@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009,2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::LoopTests;
 
@@ -14,21 +14,23 @@ our @EXPORT = qw(
 );
 
 use Test::More;
-use Test::Exception;
+use Test::Fatal;
 use Test::Refcount;
 
 use IO::Async::Test qw();
 
+use IO::File;
+use Fcntl qw( SEEK_SET );
 use POSIX qw( SIGTERM WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG );
 use Time::HiRes qw( time );
 
-our $VERSION = '0.29';
+our $VERSION = '0.45';
 
 # Abstract Units of Time
 use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
 
 # The loop under test. We keep it in a single lexical here, so we can use
-# is_oneref() tests in the individual test suite functions
+# is_oneref tests in the individual test suite functions
 my $loop;
 
 =head1 NAME
@@ -82,7 +84,7 @@ sub run_tests
    my ( $testclass, @tests ) = @_;
 
    my $count = 0;
-   $count += __PACKAGE__->can( "count_tests_$_" )->() + 2 for @tests;
+   $count += __PACKAGE__->can( "count_tests_$_" )->() + 3 for @tests;
 
    plan tests => $count;
 
@@ -94,7 +96,13 @@ sub run_tests
    }
 
    foreach my $test ( @tests ) {
-      $loop = $testclass->new();
+      $loop = $testclass->new;
+
+      is( IO::Async::Loop->new, $loop, 'magic constructor yields $loop' );
+
+      # Kill the reference in $ONE_TRUE_LOOP so as not to upset the refcounts
+      # and to ensure we get a new one each time
+      undef $IO::Async::Loop::ONE_TRUE_LOOP;
 
       is_oneref( $loop, '$loop has refcount 1' );
 
@@ -144,122 +152,227 @@ Tests the Loop's ability to watch filehandles for IO readiness
 
 =cut
 
-use constant count_tests_io => 11;
+use constant count_tests_io => 17;
 sub run_tests_io
 {
-   my ( $S1, $S2 ) = $loop->socketpair() or die "Cannot create socket pair - $!";
+   {
+      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot create socket pair - $!";
+      $_->blocking( 0 ) for $S1, $S2;
 
-   $_->blocking( 0 ) for $S1, $S2;
+      my $readready  = 0;
+      my $writeready = 0;
+      $loop->watch_io(
+         handle => $S1,
+         on_read_ready => sub { $readready = 1 },
+      );
 
-   my $readready  = 0;
-   my $writeready = 0;
+      is_oneref( $loop, '$loop has refcount 1 after watch_io on_read_ready' );
+      is( $readready, 0, '$readready still 0 before ->loop_once' );
 
-   $loop->watch_io(
-      handle => $S1,
-      on_read_ready => sub { $readready = 1 },
-   );
+      $loop->loop_once( 0.1 );
 
-   is_oneref( $loop, '$loop has refcount 1 after watch_io on_read_ready' );
-   is( $readready, 0, '$readready still 0 before ->loop_once' );
+      is( $readready, 0, '$readready when idle' );
 
-   $loop->loop_once( 0.1 );
+      $S2->syswrite( "data\n" );
 
-   is( $readready, 0, '$readready when idle' );
+      # We should still wait a little while even thought we expect to be ready
+      # immediately, because talking to ourself with 0 poll timeout is a race
+      # condition - we can still race with the kernel.
 
-   $S2->syswrite( "data\n" );
+      $loop->loop_once( 0.1 );
 
-   # We should still wait a little while even thought we expect to be ready
-   # immediately, because talking to ourself with 0 poll timeout is a race
-   # condition - we can still race with the kernel.
+      is( $readready, 1, '$readready after loop_once' );
 
-   $loop->loop_once( 0.1 );
+      # Ready $S1 to clear the data
+      $S1->getline; # ignore return
 
-   is( $readready, 1, '$readready after loop_once' );
+      $loop->watch_io(
+         handle => $S1,
+         on_write_ready => sub { $writeready = 1 },
+      );
 
-   # Ready $S1 to clear the data
-   $S1->getline(); # ignore return
+      is_oneref( $loop, '$loop has refcount 1 after watch_io on_write_ready' );
 
-   $loop->watch_io(
-      handle => $S1,
-      on_write_ready => sub { $writeready = 1 },
-   );
+      $loop->loop_once( 0.1 );
 
-   is_oneref( $loop, '$loop has refcount 1 after watch_io on_write_ready' );
+      is( $writeready, 1, '$writeready after loop_once' );
 
-   $loop->loop_once( 0.1 );
+      $loop->unwatch_io(
+         handle => $S1,
+         on_write_ready => 1,
+      );
 
-   is( $writeready, 1, '$writeready after loop_once' );
+      $readready = 0;
+      $loop->loop_once( 0.1 );
 
-   $loop->unwatch_io(
-      handle => $S1,
-      on_write_ready => 1,
-   );
+      is( $readready, 0, '$readready before HUP' );
 
-   $readready = 0;
-   $loop->loop_once( 0.1 );
+      $S2->close;
 
-   is( $readready, 0, '$readready before HUP' );
+      $readready = 0;
+      $loop->loop_once( 0.1 );
 
-   close( $S2 );
+      is( $readready, 1, '$readready after HUP' );
 
-   $readready = 0;
-   $loop->loop_once( 0.1 );
-
-   is( $readready, 1, '$readready after HUP' );
-
-   $loop->unwatch_io(
-      handle => $S1,
-      on_read_ready => 1,
-   );
+      $loop->unwatch_io(
+         handle => $S1,
+         on_read_ready => 1,
+      );
+   }
 
    # HUP of pipe - can be different to sockets on some architectures
+   {
+      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+      $_->blocking( 0 ) for $Prd, $Pwr;
 
-   my ( $P1, $P2 ) = $loop->pipepair() or die "Cannot pipepair - $!";
+      my $readready = 0;
+      $loop->watch_io(
+         handle => $Prd,
+         on_read_ready => sub { $readready = 1 },
+      );
 
-   $loop->watch_io(
-      handle => $P1,
-      on_read_ready => sub { $readready = 1 },
-   );
+      $loop->loop_once( 0.1 );
 
-   $readready = 0;
-   $loop->loop_once( 0.1 );
+      is( $readready, 0, '$readready before pipe HUP' );
 
-   is( $readready, 0, '$readready before pipe HUP' );
+      $Pwr->close;
 
-   close( $P2 );
+      $readready = 0;
+      $loop->loop_once( 0.1 );
 
-   $readready = 0;
-   $loop->loop_once( 0.1 );
+      is( $readready, 1, '$readready after pipe HUP' );
 
-   is( $readready, 1, '$readready after pipe HUP' );
+      $loop->unwatch_io(
+         handle => $Prd,
+         on_read_ready => 1,
+      );
+   }
 
-   $loop->unwatch_io(
-      handle => $P1,
-      on_read_ready => 1,
-   );
+   SKIP: {
+      $loop->_CAN_ON_HANGUP or skip "Loop cannot watch_io for on_hangup", 3;
+
+      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
+      $_->blocking( 0 ) for $S1, $S2;
+
+      my $hangup = 0;
+      $loop->watch_io(
+         handle => $S1,
+         on_hangup => sub { $hangup = 1 },
+      );
+
+      $S2->close;
+
+      $loop->loop_once( 0.1 );
+
+      is( $hangup, 1, '$hangup after socket close' );
+
+      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+      $_->blocking( 0 ) for $Prd, $Pwr;
+
+      $hangup = 0;
+      $loop->watch_io(
+         handle => $Prd,
+         on_hangup => sub { $hangup = 1 },
+      );
+
+      $Pwr->close;
+
+      $loop->loop_once( 0.1 );
+
+      is( $hangup, 1, '$hangup after pipe close for reading' );
+
+      ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+      $_->blocking( 0 ) for $Prd, $Pwr;
+
+      $hangup = 0;
+      $loop->watch_io(
+         handle => $Pwr,
+         on_hangup => sub { $hangup = 1 },
+      );
+
+      $Prd->close;
+
+      $loop->loop_once( 0.1 );
+
+      is( $hangup, 1, '$hangup after pipe close for writing' );
+   }
 
    # Check that combined read/write handlers can cancel each other
+   {
+      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
+      $_->blocking( 0 ) for $S1, $S2;
 
-   ( $S1, $S2 ) = $loop->socketpair() or die "Cannot socketpair - $!";
+      my $callcount = 0;
+      $loop->watch_io(
+         handle => $S1,
+         on_read_ready => sub {
+            $callcount++;
+            $loop->unwatch_io( handle => $S1, on_read_ready => 1, on_write_ready => 1 );
+         },
+         on_write_ready => sub {
+            $callcount++;
+            $loop->unwatch_io( handle => $S1, on_read_ready => 1, on_write_ready => 1 );
+         },
+      );
 
-   my $callcount = 0;
-   $loop->watch_io(
-      handle => $S1,
-      on_read_ready => sub {
-         $callcount++;
-         $loop->unwatch_io( handle => $S1, on_read_ready => 1, on_write_ready => 1 );
-      },
-      on_write_ready => sub {
-         $callcount++;
-         $loop->unwatch_io( handle => $S1, on_read_ready => 1, on_write_ready => 1 );
-      },
-   );
+      $S2->close;
 
-   $S2->close;
+      $loop->loop_once( 0.1 );
 
-   $loop->loop_once( 0.1 );
+      is( $callcount, 1, 'read/write_ready can cancel each other' );
+   }
 
-   is( $callcount, 1, 'read/write_ready can cancel each other' );
+   # Check that error conditions that aren't true read/write-ability are still
+   # invoked
+   {
+      my ( $S1, $S2 ) = $loop->socketpair( 'inet', 'dgram' ) or die "Cannot create AF_INET/SOCK_DGRAM connected pair - $!";
+      $_->blocking( 0 ) for $S1, $S2;
+      $S2->close;
+
+      my $readready = 0;
+      $loop->watch_io(
+         handle => $S1,
+         on_read_ready => sub { $readready = 1 },
+      );
+
+      $S1->syswrite( "Boo!" );
+
+      $loop->loop_once( 0.1 );
+
+      is( $readready, 1, 'exceptional socket invokes on_read_ready' );
+
+      $loop->unwatch_io(
+         handle => $S1,
+         on_read_ready => 1,
+      );
+   }
+
+   # Check that regular files still report read/writereadiness
+   {
+      my $F = IO::File->new_tmpfile or die "Cannot create temporary file - $!";
+
+      $F->print( "Here's some content\n" );
+      $F->seek( 0, SEEK_SET );
+
+      my $readready  = 0;
+      my $writeready = 0;
+      $loop->watch_io(
+         handle => $F,
+         on_read_ready  => sub { $readready = 1 },
+         on_write_ready => sub { $writeready = 1 },
+      );
+
+      $loop->loop_once( 0.1 );
+
+      is( $readready,  1, 'regular file is readready' );
+      is( $writeready, 1, 'regular file is writeready' );
+
+      $loop->unwatch_io(
+         handle => $F,
+         on_read_ready  => 1,
+         on_write_ready => 1,
+      );
+   }
 }
 
 =head2 timer
@@ -268,7 +381,7 @@ Tests the Loop's ability to handle timer events
 
 =cut
 
-use constant count_tests_timer => 10;
+use constant count_tests_timer => 12;
 sub run_tests_timer
 {
    my $done = 0;
@@ -281,7 +394,7 @@ sub run_tests_timer
       my $now = time;
       $loop->loop_once( 5 * AUT );
 
-      # poll() might have returned just a little early, such that the TimerQueue
+      # poll might have returned just a little early, such that the TimerQueue
       # doesn't think anything is ready yet. We need to handle that case.
       while( !$done ) {
          die "It should have been ready by now" if( time - $now > 5 * AUT );
@@ -311,7 +424,7 @@ sub run_tests_timer
       my $now = time;
       $loop->loop_once( 5 * AUT );
 
-      # poll() might have returned just a little early, such that the TimerQueue
+      # poll might have returned just a little early, such that the TimerQueue
       # doesn't think anything is ready yet. We need to handle that case.
       while( !$done ) {
          die "It should have been ready by now" if( time - $now > 5 * AUT );
@@ -326,8 +439,23 @@ sub run_tests_timer
    $done = 0;
 
    time_between {
-      $loop->loop_once() while !$done;
-   } 0, 0.1, 'loop_once() while waiting for negative interval timer';
+      $loop->loop_once while !$done;
+   } 0, 0.1, 'loop_once while waiting for negative interval timer';
+
+   my $doneA;
+   my $doneB;
+
+   $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub {
+      $loop->cancel_timer( $id ); undef $id;
+      $doneA++;
+   });
+
+   $loop->enqueue_timer( delay => 1.1 * AUT, code => sub { $doneB++ } );
+
+   $loop->loop_once( 1 * AUT ) for 1 .. 3;
+
+   is( $doneA, 1, 'Self-cancelling timer still fires' );
+   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
 }
 
 =head2 signal
@@ -336,37 +464,47 @@ Tests the Loop's ability to watch POSIX signals
 
 =cut
 
-use constant count_tests_signal => 11;
+use constant count_tests_signal => 14;
 sub run_tests_signal
 {
-   my $caught;
+   my $caught = 0;
 
-   $loop->watch_signal( TERM => sub { $caught = 1 } );
+   $loop->watch_signal( TERM => sub { $caught++ } );
 
-   is_oneref( $loop, '$loop has refcount 1 after watch_signal()' );
+   is_oneref( $loop, '$loop has refcount 1 after watch_signal' );
 
    $loop->loop_once( 0.1 );
 
-   is( $caught, undef, '$caught idling' );
+   is( $caught, 0, '$caught idling' );
 
    kill SIGTERM, $$;
 
+   is( $caught, 0, '$caught before ->loop_once' );
+
    $loop->loop_once( 0.1 );
 
-   is( $caught, 1, '$caught after raise' );
+   is( $caught, 1, '$caught after ->loop_once' );
 
-   is_oneref( $loop, '$loop has refcount 1 before unwatch_signal()' );
+   kill SIGTERM, $$;
+
+   is( $caught, 1, 'second raise is still deferred' );
+
+   $loop->loop_once( 0.1 );
+
+   is( $caught, 2, '$caught after second ->loop_once' );
+
+   is_oneref( $loop, '$loop has refcount 1 before unwatch_signal' );
 
    $loop->unwatch_signal( 'TERM' );
 
-   is_oneref( $loop, '$loop has refcount 1 after unwatch_signal()' );
+   is_oneref( $loop, '$loop has refcount 1 after unwatch_signal' );
 
    my ( $cA, $cB );
 
    my $idA = $loop->attach_signal( TERM => sub { $cA = 1 } );
    my $idB = $loop->attach_signal( TERM => sub { $cB = 1 } );
 
-   is_oneref( $loop, '$loop has refcount 1 after 2 * attach_signal()' );
+   is_oneref( $loop, '$loop has refcount 1 after 2 * attach_signal' );
 
    kill SIGTERM, $$;
 
@@ -389,8 +527,8 @@ sub run_tests_signal
 
    $loop->detach_signal( 'TERM', $idB );
 
-   dies_ok( sub { $loop->attach_signal( 'this signal name does not exist', sub {} ) },
-            'Bad signal name fails' );
+   ok( exception { $loop->attach_signal( 'this signal name does not exist', sub {} ) },
+       'Bad signal name fails' );
 }
 
 =head2 idle
@@ -404,7 +542,7 @@ sub run_tests_idle
 {
    my $called = 0;
 
-   my $id = $loop->watch_idle( when => 'later', code => sub { $called++ } );
+   my $id = $loop->watch_idle( when => 'later', code => sub { $called = 1 } );
 
    ok( defined $id, 'idle watcher id is defined' );
 
@@ -415,7 +553,7 @@ sub run_tests_idle
    is( $called, 1, 'deferred sub called after loop_once' );
 
    $loop->watch_idle( when => 'later', code => sub {
-      $loop->watch_idle( when => 'later', code => sub { $called++ } )
+      $loop->watch_idle( when => 'later', code => sub { $called = 2 } )
    } );
 
    $loop->loop_once( 1 );
@@ -426,6 +564,8 @@ sub run_tests_idle
 
    is( $called, 2, 'inner deferral now invoked' );
 
+   $called = 2; # set it anyway in case previous test fails
+
    $id = $loop->watch_idle( when => 'later', code => sub { $called = 20 } );
 
    $loop->unwatch_idle( $id );
@@ -434,11 +574,11 @@ sub run_tests_idle
 
    is( $called, 2, 'unwatched deferral not called' );
 
-   $loop->later( sub { $called++ } );
+   $loop->later( sub { $called = 3 } );
 
    $loop->loop_once( 1 );
 
-   is( $called, 3, '$loop->later() shortcut works' );
+   is( $called, 3, '$loop->later shortcut works' );
 }
 
 =head2 child
@@ -447,15 +587,22 @@ Tests the Loop's support for watching child processes by PID
 
 =cut
 
-use constant count_tests_child => 6;
+sub run_in_child(&)
+{
+   my $kid = fork;
+   defined $kid or die "Cannot fork() - $!";
+   return $kid if $kid;
+
+   shift->();
+   die "Fell out of run_in_child!\n";
+}
+
+use constant count_tests_child => 8;
 sub run_tests_child
 {
-   my $kid = fork();
-   defined $kid or die "Cannot fork() - $!";
-
-   if( $kid == 0 ) {
+   my $kid = run_in_child {
       exit( 3 );
-   }
+   };
 
    my $exitcode;
 
@@ -475,14 +622,11 @@ sub run_tests_child
    # ignored or handled elsewhere.
    local $SIG{TERM} = "DEFAULT";
 
-   $kid = fork();
-   defined $kid or die "Cannot fork() - $!";
-
-   if( $kid == 0 ) {
+   $kid = run_in_child {
       sleep( 10 );
       # Just in case the parent died already and didn't kill us
       exit( 0 );
-   }
+   };
 
    $loop->watch_child( $kid => sub { ( undef, $exitcode ) = @_; } );
 
@@ -493,6 +637,17 @@ sub run_tests_child
 
    ok( WIFSIGNALED($exitcode),          'WIFSIGNALED($exitcode) after SIGTERM' );
    is( WTERMSIG($exitcode),    SIGTERM, 'WTERMSIG($exitcode) after SIGTERM' );
+
+   my %kids;
+
+   $loop->watch_child( 0 => sub { my ( $kid ) = @_; delete $kids{$kid} } );
+
+   %kids = map { run_in_child { exit 0 } => 1 } 1 .. 3;
+
+   is( scalar keys %kids, 3, 'Waiting for 3 child processes' );
+
+   wait_for { !keys %kids };
+   ok( !keys %kids, 'All child processes reclaimed' );
 }
 
 =head2 control
@@ -501,12 +656,14 @@ Tests that the C<loop_once> and C<loop_forever> methods behave correctly
 
 =cut
 
-use constant count_tests_control => 3;
+use constant count_tests_control => 5;
 sub run_tests_control
 {
+   time_between { $loop->loop_once( 0 ) } 0, 0.1, 'loop_once(0) when idle';
+
    time_between { $loop->loop_once( 2 * AUT ) } 1.5, 2.5, 'loop_once(2) when idle';
 
-   $loop->later( sub { $loop->loop_stop } );
+   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->loop_stop } );
 
    local $SIG{ALRM} = sub { die "Test timed out before ->loop_stop" };
    alarm( 1 );
@@ -518,11 +675,10 @@ sub run_tests_control
    ok( 1, '$loop->loop_forever interruptable by ->loop_stop' );
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
