@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.42';
+our $VERSION = '0.45';
 
 use Carp;
 
@@ -22,7 +22,7 @@ C<IO::Async::Timer::Countdown> - event callback after a fixed delay
  use IO::Async::Timer::Countdown;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  my $timer = IO::Async::Timer::Countdown->new(
     delay => 10,
@@ -79,6 +79,11 @@ CODE reference for the C<on_expire> event.
 The delay in seconds after starting the timer until it expires. Cannot be
 changed if the timer is running.
 
+=item remove_on_expire => BOOL
+
+Optional. If true, remove this timer object from its parent notifier or
+containing loop when it expires. Defaults to false.
+
 =back
 
 Once constructed, the timer object will need to be added to the C<Loop> before
@@ -91,6 +96,10 @@ sub configure
    my $self = shift;
    my %params = @_;
 
+   foreach (qw( remove_on_expire )) {
+      $self->{$_} = delete $params{$_} if exists $params{$_};
+   }
+
    if( exists $params{on_expire} ) {
       my $on_expire = delete $params{on_expire};
       ref $on_expire or croak "Expected 'on_expire' as a reference";
@@ -115,6 +124,22 @@ sub configure
    $self->SUPER::configure( %params );
 }
 
+=head1 METHODS
+
+=cut
+
+=head2 $expired = $timer->is_expired
+
+Returns true if the Timer has already expired.
+
+=cut
+
+sub is_expired
+{
+   my $self = shift;
+   return $self->{expired};
+}
+
 sub _make_cb
 {
    my $self = shift;
@@ -123,6 +148,9 @@ sub _make_cb
       my ( $self ) = @_;
 
       undef $self->{id};
+      $self->{expired} = 1;
+
+      $self->remove_from_parent if $self->{remove_on_expire};
 
       $self->invoke_event( "on_expire" );
    } );
@@ -132,6 +160,7 @@ sub _make_enqueueargs
 {
    my $self = shift;
 
+   undef $self->{expired};
    return delay => $self->{delay};
 }
 
@@ -146,7 +175,7 @@ sub reset
 {
    my $self = shift;
 
-   my $loop = $self->get_loop or croak "Cannot reset a Timer that is not in a Loop";
+   my $loop = $self->loop or croak "Cannot reset a Timer that is not in a Loop";
 
    return if !$self->is_running;
 
